@@ -86,7 +86,8 @@ gst_ffmpeg_mb_decision_get_type (void)
   if (!ffmpeg_mb_decision_type) {
     static const GEnumValue ffmpeg_mb_decisions[] = {
       {FF_MB_DECISION_SIMPLE, "Use method set by mb-cmp", "simple"},
-      {FF_MB_DECISION_BITS, "Chooses the one which needs the fewest bits aka vhq mode", "bits"},
+      {FF_MB_DECISION_BITS,
+          "Chooses the one which needs the fewest bits aka vhq mode", "bits"},
       {FF_MB_DECISION_RD, "Rate Distortion", "rd"},
       {0, NULL, NULL},
     };
@@ -222,6 +223,7 @@ gst_ffmpeg_pre_me_get_type (void)
       {0, "Disabled", "off"},
       {1, "Only after I-frames", "key"},
       {2, "Always", "all"},
+      {0, NULL, NULL}
     };
 
     ffmpeg_pre_me_type =
@@ -242,6 +244,7 @@ gst_ffmpeg_pred_method_get_type (void)
       {FF_PRED_LEFT, "Left", "left"},
       {FF_PRED_PLANE, "Plane", "plane"},
       {FF_PRED_MEDIAN, "Median", "median"},
+      {0, NULL, NULL}
     };
 
     ffmpeg_pred_method =
@@ -260,18 +263,28 @@ gst_ffmpeg_flags_get_type (void)
   if (!ffmpeg_flags_type) {
     static const GFlagsValue ffmpeg_flags[] = {
       {CODEC_FLAG_4MV, "Allow 4 MV per MB", "4mv"},
+      {CODEC_FLAG_H263P_AIV, "H.263 alternative inter VLC", "aiv"},
       {CODEC_FLAG_QPEL, "Quartel Pel Motion Compensation", "qpel"},
       {CODEC_FLAG_GMC, "GMC", "gmc"},
       {CODEC_FLAG_MV0, "Always try a MB with MV (0,0)", "mv0"},
-      {CODEC_FLAG_PART, "Store MV, DC and AC coefficients in seperate partitions", "part"},
+      {CODEC_FLAG_PART,
+          "Store MV, DC and AC coefficients in seperate partitions", "part"},
       {CODEC_FLAG_GRAY, "Only decode/encode grayscale", "gray"},
-      {CODEC_FLAG_NORMALIZE_AQP, "Normalize Adaptive Quantization (masking, etc)", "aqp"},
+      {CODEC_FLAG_NORMALIZE_AQP,
+          "Normalize Adaptive Quantization (masking, etc)", "aqp"},
       {CODEC_FLAG_TRELLIS_QUANT, "Trellis Quantization", "trellis"},
-      {CODEC_FLAG_AC_PRED, "H263 Advanced Intra Coding / MPEG4 AC prediction", "aic"},
+      {CODEC_FLAG_GLOBAL_HEADER,
+            "Global headers in extradata instead of every keyframe",
+          "global-headers"},
+      {CODEC_FLAG_AC_PRED, "H263 Advanced Intra Coding / MPEG4 AC prediction",
+          "aic"},
       {CODEC_FLAG_H263P_UMV, "Unlimited Motion Vector", "umv"},
       {CODEC_FLAG_CBP_RD, "Rate Distoration Optimization for CBP", "cbp-rd"},
-      {CODEC_FLAG_QP_RD, "Rate Distoration Optimization for QP selection", "qp-rd"},
-      {CODEC_FLAG_SVCD_SCAN_OFFSET, "Reserve space for SVC scan offset user data", "scanoffset"},
+      {CODEC_FLAG_QP_RD, "Rate Distoration Optimization for QP selection",
+          "qp-rd"},
+      {CODEC_FLAG_H263P_SLICE_STRUCT, "H263 slice struct", "ss"},
+      {CODEC_FLAG_SVCD_SCAN_OFFSET,
+          "Reserve space for SVCD scan offset user data", "scanoffset"},
       {CODEC_FLAG_CLOSED_GOP, "Closed GOP", "closedgop"},
       {0, NULL, NULL},
     };
@@ -340,19 +353,15 @@ G_STMT_START {                                                          \
 /* ==== BEGIN CONFIGURATION SECTION ==== */
 
 /* some typical include and exclude lists; modify and/or add where needed */
-#if 0
-static gint mpeg12[] = {
-  CODEC_ID_MPEG1VIDEO,
-  CODEC_ID_MPEG2VIDEO,
-  CODEC_ID_NONE
-};
-#endif
 
-static gint mpeg4[] = {
+static gint mpeg[] = {
   CODEC_ID_MPEG4,
   CODEC_ID_MSMPEG4V1,
   CODEC_ID_MSMPEG4V2,
   CODEC_ID_MSMPEG4V3,
+  CODEC_ID_MPEG1VIDEO,
+  CODEC_ID_MPEG2VIDEO,
+  CODEC_ID_H263P,
   CODEC_ID_FLV1,
   CODEC_ID_NONE
 };
@@ -379,302 +388,282 @@ gst_ffmpeg_cfg_init ()
 
   /* list properties here */
   pspec = g_param_spec_enum ("pass", "Encoding pass/type",
-      "Encoding pass/type", GST_TYPE_FFMPEG_PASS,
-      0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, pass, FALSE, mpeg4, NULL);
+      "Encoding pass/type", GST_TYPE_FFMPEG_PASS, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, pass, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("quantizer", "Constant Quantizer",
-      "Constant Quantizer", 0, 30, 0.01f,
-      G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, quantizer, FALSE, mpeg4, NULL);
+      "Constant Quantizer", 0, 30, 0.01f, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, quantizer, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_string ("statsfile", "Statistics Filename",
       "Filename to store data for 2-pass encoding", "stats.log",
       G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, filename, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, filename, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("bitrate-tolerance", "Bitrate Tolerance",
       "Number of bits the bitstream is allowed to diverge from the reference",
       0, 100000000, 8000000, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.bit_rate_tolerance, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.bit_rate_tolerance, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("mb-decision", "Macroblock Decision",
       "Macroblok Decision Mode",
       GST_TYPE_FFMPEG_MB_DECISION, FF_CMP_SAD, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.mb_decision, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.mb_decision, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("mb-cmp", "Macroblock Compare Function",
       "Macroblok Compare Function",
       GST_TYPE_FFMPEG_CMP_FUNCTION, FF_CMP_SAD, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.mb_cmp, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.mb_cmp, FALSE, mpeg, NULL);
 
-  pspec = g_param_spec_enum ("me-pre-cmp", "Motion Estimation Pre Pass Compare Function",
+  pspec =
+      g_param_spec_enum ("me-pre-cmp",
+      "Motion Estimation Pre Pass Compare Function",
       "Motion Estimation Pre Pass Compare Function",
       GST_TYPE_FFMPEG_CMP_FUNCTION, FF_CMP_SAD, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.me_pre_cmp, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.me_pre_cmp, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("me-cmp", "Motion Estimation Compare Function",
       "Motion Estimation Compare Function",
       GST_TYPE_FFMPEG_CMP_FUNCTION, FF_CMP_SAD, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.me_cmp, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.me_cmp, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("me-sub-cmp",
       "Subpixel Motion Estimation Compare Function",
       "Subpixel Motion Estimation Compare Function",
       GST_TYPE_FFMPEG_CMP_FUNCTION, FF_CMP_SAD, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.me_sub_cmp, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.me_sub_cmp, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("ildct-cmp", "Interlaced DCT Compare Function",
       "Interlaced DCT Compare Function",
       GST_TYPE_FFMPEG_CMP_FUNCTION, FF_CMP_VSAD, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.ildct_cmp, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.ildct_cmp, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("dct-algo", "DCT Algorithm",
       "DCT Algorithm",
       GST_TYPE_FFMPEG_DCT_ALGO, FF_DCT_AUTO, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.dct_algo, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.dct_algo, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("idct-algo", "IDCT Algorithm",
       "IDCT Algorithm",
       GST_TYPE_FFMPEG_IDCT_ALGO, FF_IDCT_AUTO, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.idct_algo, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.idct_algo, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("quant-type", "Quantizer Type",
-      "Quantizer Type",
-      GST_TYPE_FFMPEG_QUANT_TYPE, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.mpeg_quant, FALSE, mpeg4, NULL);
+      "Quantizer Type", GST_TYPE_FFMPEG_QUANT_TYPE, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.mpeg_quant, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("qmin", "Minimum Quantizer",
-      "Minimum Quantizer",
-      1, 31, 2, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.qmin, FALSE, mpeg4, NULL);
+      "Minimum Quantizer", 1, 31, 2, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.qmin, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("qmax", "Maximum Quantizer",
-      "Maximum Quantizer",
-      1, 31, 31, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.qmax, FALSE, mpeg4, NULL);
+      "Maximum Quantizer", 1, 31, 31, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.qmax, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("max-qdiff", "Maximum Quantizer Difference",
       "Maximum Quantizer Difference between frames",
       1, 31, 3, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.max_qdiff, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.max_qdiff, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("mb_qmin", "Minimum MB Quantizer",
-      "Minimum MB Quantizer",
-      0, 31, 2, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.mb_qmin, FALSE, mpeg4, NULL);
+      "Minimum MB Quantizer", 0, 31, 2, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.mb_qmin, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("mb_qmax", "Maximum MB Quantizer",
-      "Maximum MB Quantizer",
-      0, 31, 31, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.mb_qmax, FALSE, mpeg4, NULL);
+      "Maximum MB Quantizer", 0, 31, 31, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.mb_qmax, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("lmin", "Minimum Lagrange Multiplier",
-      "Minimum Lagrange Multiplier",
-      1, 31, 2, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, lmin, FALSE, mpeg4, NULL);
+      "Minimum Lagrange Multiplier", 1, 31, 2, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, lmin, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("lmax", "Maximum Lagrange Multiplier",
-      "Maximum Lagrange Multiplier",
-      1, 31, 31, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, lmax, FALSE, mpeg4, NULL);
+      "Maximum Lagrange Multiplier", 1, 31, 31, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, lmax, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("qcompress", "Quantizer Change",
       "Quantizer Change between easy and hard scenes",
       0, 1.0f, 0.5f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.qcompress, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.qcompress, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("qblur", "Quantizer Smoothing",
-      "Quantizer Smoothing over time",
-      0, 1.0f, 0.5f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.qblur, FALSE, mpeg4, NULL);
+      "Quantizer Smoothing over time", 0, 1.0f, 0.5f, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.qblur, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("rc-qsquish", "Ratecontrol Limiting Method",
       "0 means limit by clipping, otherwise use nice continuous function",
       0, 99.0f, 1.0f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_qsquish, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.rc_qsquish, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("rc-qmod-amp", "Ratecontrol Mod",
-      "Ratecontrol Mod",
-      0, 99.0f, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_qmod_amp, FALSE, mpeg4, NULL);
+      "Ratecontrol Mod", 0, 99.0f, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.rc_qmod_amp, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("rc-qmod-freq", "Ratecontrol Freq",
-      "Ratecontrol Freq",
-      0, 0, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_qmod_freq, FALSE, mpeg4, NULL);
+      "Ratecontrol Freq", 0, 0, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.rc_qmod_freq, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("rc-buffer-size", "Ratecontrol Buffer Size",
-      "Decoder bitstream buffer size",
-      0, G_MAXINT, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_buffer_size, FALSE, mpeg4, NULL);
+      "Decoder bitstream buffer size", 0, G_MAXINT, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.rc_buffer_size, FALSE, mpeg, NULL);
 
-  pspec = g_param_spec_float ("rc-buffer-aggressivity", "Ratecontrol Buffer Aggressivity",
-      "Ratecontrol Buffer Aggressivity",
-      0, 99.0f, 1.0f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_buffer_aggressivity, FALSE, mpeg4, NULL);
+  pspec =
+      g_param_spec_float ("rc-buffer-aggressivity",
+      "Ratecontrol Buffer Aggressivity", "Ratecontrol Buffer Aggressivity", 0,
+      99.0f, 1.0f, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.rc_buffer_aggressivity, FALSE, mpeg,
+      NULL);
 
   pspec = g_param_spec_int ("rc-max-rate", "Ratecontrol Maximum Bitrate",
-      "Ratecontrol Maximum Bitrate",
-      0, G_MAXINT, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_max_rate, FALSE, mpeg4, NULL);
+      "Ratecontrol Maximum Bitrate", 0, G_MAXINT, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.rc_max_rate, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("rc-min-rate", "Ratecontrol Minimum Bitrate",
-      "Ratecontrol Minimum Bitrate",
-      0, G_MAXINT, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_min_rate, FALSE, mpeg4, NULL);
+      "Ratecontrol Minimum Bitrate", 0, G_MAXINT, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.rc_min_rate, FALSE, mpeg, NULL);
 
-  pspec = g_param_spec_float ("rc-initial-cplx", "Initial Complexity for Pass 1 Ratecontrol",
+  pspec =
+      g_param_spec_float ("rc-initial-cplx",
       "Initial Complexity for Pass 1 Ratecontrol",
-      0, 9999999.0f, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_initial_cplx, FALSE, mpeg4, NULL);
+      "Initial Complexity for Pass 1 Ratecontrol", 0, 9999999.0f, 0,
+      G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.rc_initial_cplx, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_string ("rc-eq", "Ratecontrol Equation",
-      "Ratecontrol Equation",
-      "tex^qComp", G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.rc_eq, FALSE, mpeg4, NULL);
+      "Ratecontrol Equation", "tex^qComp", G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.rc_eq, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("b-quant-factor", "B-Quantizer Factor",
       "Factor in B-Frame Quantizer Computation",
       -31.0f, 31.0f, 1.25f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.b_quant_factor, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.b_quant_factor, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("b-quant-offset", "B-Quantizer Offset",
       "Offset in B-Frame Quantizer Computation",
       0.0f, 31.0f, 1.25f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.b_quant_offset, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.b_quant_offset, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("i-quant-factor", "I-Quantizer Factor",
       "Factor in P-Frame Quantizer Computation",
       -31.0f, 31.0f, 0.8f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.i_quant_factor, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.i_quant_factor, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("i-quant-offset", "I-Quantizer Offset",
       "Offset in P-Frame Quantizer Computation",
       0.0f, 31.0f, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.i_quant_offset, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.i_quant_offset, FALSE, mpeg, NULL);
 
   /* note overlap with gop-size; 0 means do not override */
   pspec = g_param_spec_int ("max-key-interval", "Maximum Key Interval",
       "Maximum number of frames between two keyframes (< 0 is in sec)",
       -100, G_MAXINT, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, max_key_interval, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, max_key_interval, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("luma-elim-threshold",
       "Luma Elimination Threshold",
       "Luma Single Coefficient Elimination Threshold",
       -99, 99, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.luma_elim_threshold, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.luma_elim_threshold, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("chroma-elim-threshold",
       "Chroma Elimination Threshold",
       "Chroma Single Coefficient Elimination Threshold",
       -99, 99, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.chroma_elim_threshold, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.chroma_elim_threshold, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("lumi-masking", "Luminance Masking",
-      "Luminance Masking",
-      -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.lumi_masking, FALSE, mpeg4, NULL);
+      "Luminance Masking", -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.lumi_masking, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("dark-masking", "Darkness Masking",
-      "Darkness Masking",
-      -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.dark_masking, FALSE, mpeg4, NULL);
+      "Darkness Masking", -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.dark_masking, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("temporal-cplx-masking",
       "Temporal Complexity Masking",
-      "Temporal Complexity Masking",
-      -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.temporal_cplx_masking, FALSE, mpeg4, NULL);
+      "Temporal Complexity Masking", -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.temporal_cplx_masking, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("spatial-cplx-masking",
       "Spatial Complexity Masking",
-      "Spatial Complexity Masking",
-      -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.spatial_cplx_masking, FALSE, mpeg4, NULL);
+      "Spatial Complexity Masking", -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.spatial_cplx_masking, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_float ("p-masking", "P Block Masking",
-      "P Block  Masking",
-      -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.p_masking, FALSE, mpeg4, NULL);
+      "P Block  Masking", -1.0f, 1.0f, 0.0f, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.p_masking, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("dia-size",
       "Motion Estimation Diamond Size/Shape",
       "Motion Estimation Diamond Size/Shape",
       -2000, 2000, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.dia_size, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.dia_size, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("pre-dia-size",
       "Motion Estimation Pre Pass Diamond Size/Shape",
       "Motion Estimation Diamond Size/Shape",
       -2000, 2000, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.pre_dia_size, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.pre_dia_size, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("last-predictor-count",
       "Last Predictor Count",
       "Amount of previous Motion Vector predictors",
-    0, 2000, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.last_predictor_count, FALSE, mpeg4, NULL);
+      0, 2000, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.last_predictor_count, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("pre-me",
       "Pre Pass for Motion Estimation",
       "Pre Pass for Motion Estimation",
       GST_TYPE_FFMPEG_PRE_ME, 1, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.pre_me, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.pre_me, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("me-subpel-quality",
       "Motion Estimation Subpixel Quality",
       "Motion Estimation Subpixel Refinement Quality",
       0, 8, 8, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.me_subpel_quality, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.me_subpel_quality, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("me-range",
       "Motion Estimation Range",
       "Motion Estimation search range in subpel units",
       0, 16000, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.me_range, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.me_range, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("intra-quant-bias",
       "Intra Quantizer Bias",
       "Intra Quantizer Bias",
       -1000000, 1000000, FF_DEFAULT_QUANT_BIAS, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.intra_quant_bias, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.intra_quant_bias, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("inter-quant-bias",
       "Inter Quantizer Bias",
       "Inter Quantizer Bias",
       -1000000, 1000000, FF_DEFAULT_QUANT_BIAS, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.inter_quant_bias, FALSE, mpeg4, NULL);
+  gst_ffmpeg_add_pspec (pspec, config.inter_quant_bias, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("noise-reduction",
       "Noise Reduction",
-      "Noise Reduction Strength",
-      0, 1000000, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.noise_reduction, FALSE, mpeg4, NULL);
+      "Noise Reduction Strength", 0, 1000000, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.noise_reduction, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("intra-dc-precision",
       "Intra DC precision",
-      "Precision of the Intra DC coefficient - 8",
-      0, 16, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.intra_dc_precision, FALSE, mpeg4, NULL);
+      "Precision of the Intra DC coefficient - 8", 0, 16, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.intra_dc_precision, FALSE, mpeg, NULL);
 
   /* TODO skipped coder_type, context_model, inter_threshold, scenechange_threshold */
 
   pspec = g_param_spec_flags ("flags", "Flags",
-      "Flags",
-      GST_TYPE_FFMPEG_FLAGS, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.flags, FALSE, mpeg4, NULL);
+      "Flags", GST_TYPE_FFMPEG_FLAGS, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.flags, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_boolean ("interlaced", "Interlaced Material",
-      "Interlaced Material",
-      FALSE, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, interlaced, FALSE, mpeg4, NULL);
+      "Interlaced Material", FALSE, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, interlaced, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_int ("max-bframes", "Max B-Frames",
-      "Maximum B-frames in a row",
-      0, FF_MAX_B_FRAMES, 0, G_PARAM_READWRITE);
-  gst_ffmpeg_add_pspec (pspec, config.max_b_frames, FALSE, mpeg4, NULL);
+      "Maximum B-frames in a row", 0, FF_MAX_B_FRAMES, 0, G_PARAM_READWRITE);
+  gst_ffmpeg_add_pspec (pspec, config.max_b_frames, FALSE, mpeg, NULL);
 
   pspec = g_param_spec_enum ("prediction-method", "Prediction Method",
       "Prediction Method",
@@ -687,17 +676,17 @@ gst_ffmpeg_cfg_init ()
 
 /* return TRUE if property described by pspec applies to the codec with codec_id */
 static gboolean
-gst_ffmpeg_cfg_codec_has_pspec (enum CodecID codec_id, GParamSpec *pspec)
+gst_ffmpeg_cfg_codec_has_pspec (enum CodecID codec_id, GParamSpec * pspec)
 {
   GParamSpecData *qdata;
-  gint* codec;
+  gint *codec;
   gboolean ret = FALSE;
 
   qdata = g_param_spec_get_qdata (pspec, quark);
 
   /* check if excluded first */
   if ((codec = qdata->exclude_list)) {
-    for ( ; *codec != CODEC_ID_NONE; ++codec) {
+    for (; *codec != CODEC_ID_NONE; ++codec) {
       if (*codec == codec_id)
         return FALSE;
     }
@@ -705,7 +694,7 @@ gst_ffmpeg_cfg_codec_has_pspec (enum CodecID codec_id, GParamSpec *pspec)
 
   /* no include list means it is accepted */
   if ((codec = qdata->include_list)) {
-    for ( ; *codec != CODEC_ID_NONE; ++codec) {
+    for (; *codec != CODEC_ID_NONE; ++codec) {
       if (*codec == codec_id)
         ret = TRUE;
     }
@@ -728,7 +717,7 @@ gst_ffmpeg_cfg_install_property (GstFFMpegEncClass * klass, guint base)
   prop_id = base;
   g_return_if_fail (base > 0);
 
-  ctx = avcodec_alloc_context();
+  ctx = avcodec_alloc_context ();
   if (ctx)
     avcodec_get_context_defaults (ctx);
   else
@@ -740,9 +729,9 @@ gst_ffmpeg_cfg_install_property (GstFFMpegEncClass * klass, guint base)
       /* 'clone' the paramspec for the various codecs,
        * since a single paramspec cannot be owned by distinct types */
 
-      const gchar* name = g_param_spec_get_name (pspec);
-      const gchar* nick = g_param_spec_get_nick (pspec);
-      const gchar* blurb = g_param_spec_get_blurb (pspec);
+      const gchar *name = g_param_spec_get_name (pspec);
+      const gchar *nick = g_param_spec_get_nick (pspec);
+      const gchar *blurb = g_param_spec_get_blurb (pspec);
       GParamSpecData *qdata = g_param_spec_get_qdata (pspec, quark);
       gint ctx_offset = 0;
       gboolean lavc_default;
@@ -760,71 +749,73 @@ gst_ffmpeg_cfg_install_property (GstFFMpegEncClass * klass, guint base)
       }
 
       switch (G_PARAM_SPEC_VALUE_TYPE (pspec)) {
-        case G_TYPE_STRING: {
-          GParamSpecString* pstring = G_PARAM_SPEC_STRING (pspec);
+        case G_TYPE_STRING:{
+          GParamSpecString *pstring = G_PARAM_SPEC_STRING (pspec);
+
           pspec = g_param_spec_string (name, nick, blurb,
-              lavc_default ? G_STRUCT_MEMBER (gchar*, ctx, ctx_offset)
-                : pstring->default_value, 
-              pspec->flags);
+              lavc_default ? G_STRUCT_MEMBER (gchar *, ctx, ctx_offset)
+              : pstring->default_value, pspec->flags);
           break;
         }
-        case G_TYPE_INT: {
-          GParamSpecInt* pint = G_PARAM_SPEC_INT (pspec);
+        case G_TYPE_INT:{
+          GParamSpecInt *pint = G_PARAM_SPEC_INT (pspec);
+
           pspec = g_param_spec_int (name, nick, blurb,
               pint->minimum, pint->maximum,
               lavc_default ? G_STRUCT_MEMBER (gint, ctx, ctx_offset)
-                : pint->default_value,
-              pspec->flags);
+              : pint->default_value, pspec->flags);
           break;
         }
-        case G_TYPE_UINT: {
-          GParamSpecUInt* puint = G_PARAM_SPEC_UINT (pspec);
+        case G_TYPE_UINT:{
+          GParamSpecUInt *puint = G_PARAM_SPEC_UINT (pspec);
+
           pspec = g_param_spec_uint (name, nick, blurb,
-                puint->minimum, puint->maximum,
-                lavc_default ? G_STRUCT_MEMBER (guint, ctx, ctx_offset)
-                : puint->default_value,
-              pspec->flags);
+              puint->minimum, puint->maximum,
+              lavc_default ? G_STRUCT_MEMBER (guint, ctx, ctx_offset)
+              : puint->default_value, pspec->flags);
           break;
         }
-        case G_TYPE_ULONG: {
-          GParamSpecULong* pulong = G_PARAM_SPEC_ULONG (pspec);
+        case G_TYPE_ULONG:{
+          GParamSpecULong *pulong = G_PARAM_SPEC_ULONG (pspec);
+
           pspec = g_param_spec_ulong (name, nick, blurb,
               pulong->minimum, pulong->maximum,
               lavc_default ? G_STRUCT_MEMBER (gulong, ctx, ctx_offset)
-                : pulong->default_value,
-              pspec->flags);
+              : pulong->default_value, pspec->flags);
           break;
         }
-        case G_TYPE_FLOAT: {
-          GParamSpecFloat* pfloat = G_PARAM_SPEC_FLOAT (pspec);
+        case G_TYPE_FLOAT:{
+          GParamSpecFloat *pfloat = G_PARAM_SPEC_FLOAT (pspec);
+
           pspec = g_param_spec_float (name, nick, blurb,
               pfloat->minimum, pfloat->maximum,
               lavc_default ? G_STRUCT_MEMBER (gfloat, ctx, ctx_offset)
-                : pfloat->default_value,
-              pspec->flags);
+              : pfloat->default_value, pspec->flags);
           break;
         }
-        case G_TYPE_BOOLEAN: {
-          GParamSpecBoolean* pboolean = G_PARAM_SPEC_BOOLEAN (pspec);
+        case G_TYPE_BOOLEAN:{
+          GParamSpecBoolean *pboolean = G_PARAM_SPEC_BOOLEAN (pspec);
+
           pspec = g_param_spec_boolean (name, nick, blurb,
               lavc_default ? G_STRUCT_MEMBER (gboolean, ctx, ctx_offset)
-                : pboolean->default_value,
-              pspec->flags);
+              : pboolean->default_value, pspec->flags);
           break;
         }
         default:
           if (G_IS_PARAM_SPEC_ENUM (pspec)) {
-            GParamSpecEnum* penum = G_PARAM_SPEC_ENUM (pspec);
+            GParamSpecEnum *penum = G_PARAM_SPEC_ENUM (pspec);
+
             pspec = g_param_spec_enum (name, nick, blurb,
                 pspec->value_type,
-                lavc_default ? G_STRUCT_MEMBER (gint, ctx, ctx_offset) 
-                  : penum->default_value, pspec->flags);
+                lavc_default ? G_STRUCT_MEMBER (gint, ctx, ctx_offset)
+                : penum->default_value, pspec->flags);
           } else if (G_IS_PARAM_SPEC_FLAGS (pspec)) {
-            GParamSpecFlags* pflags = G_PARAM_SPEC_FLAGS (pspec);
+            GParamSpecFlags *pflags = G_PARAM_SPEC_FLAGS (pspec);
+
             pspec = g_param_spec_flags (name, nick, blurb,
                 pspec->value_type,
-                lavc_default ? G_STRUCT_MEMBER (guint, ctx, ctx_offset) 
-                  : pflags->default_value, pspec->flags);
+                lavc_default ? G_STRUCT_MEMBER (guint, ctx, ctx_offset)
+                : pflags->default_value, pspec->flags);
           } else {
             g_critical ("%s does not yet support type %s", GST_FUNCTION,
                 g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
@@ -885,22 +876,22 @@ gst_ffmpeg_cfg_set_property (GObject * object,
           g_value_get_float (value);
       break;
     case G_TYPE_STRING:
-      g_return_val_if_fail (qdata->size == sizeof (gchar*), TRUE);
-      g_free (G_STRUCT_MEMBER (gchar*, ffmpegenc, qdata->offset));
-      G_STRUCT_MEMBER (gchar*, ffmpegenc, qdata->offset) =
+      g_return_val_if_fail (qdata->size == sizeof (gchar *), TRUE);
+      g_free (G_STRUCT_MEMBER (gchar *, ffmpegenc, qdata->offset));
+      G_STRUCT_MEMBER (gchar *, ffmpegenc, qdata->offset) =
           g_value_dup_string (value);
       break;
-    default: /* must be enum, given the check above */
+    default:                   /* must be enum, given the check above */
       if (G_IS_PARAM_SPEC_ENUM (pspec)) {
         g_return_val_if_fail (qdata->size == sizeof (gint), TRUE);
         G_STRUCT_MEMBER (gint, ffmpegenc, qdata->offset) =
-          g_value_get_enum (value);
+            g_value_get_enum (value);
       } else if (G_IS_PARAM_SPEC_FLAGS (pspec)) {
         g_return_val_if_fail (qdata->size == sizeof (guint), TRUE);
         G_STRUCT_MEMBER (guint, ffmpegenc, qdata->offset) =
             g_value_get_flags (value);
-      } else { /* oops, bit lazy we don't cover this case yet */
-        g_critical ("%s does not yet support type %s",GST_FUNCTION,
+      } else {                  /* oops, bit lazy we don't cover this case yet */
+        g_critical ("%s does not yet support type %s", GST_FUNCTION,
             g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
       }
 
@@ -939,8 +930,7 @@ gst_ffmpeg_cfg_get_property (GObject * object,
       break;
     case G_TYPE_INT:
       g_return_val_if_fail (qdata->size == sizeof (gint), TRUE);
-      g_value_set_int (value,
-          G_STRUCT_MEMBER (gint, ffmpegenc, qdata->offset));
+      g_value_set_int (value, G_STRUCT_MEMBER (gint, ffmpegenc, qdata->offset));
       break;
     case G_TYPE_ULONG:
       g_return_val_if_fail (qdata->size == sizeof (gulong), TRUE);
@@ -953,11 +943,11 @@ gst_ffmpeg_cfg_get_property (GObject * object,
           G_STRUCT_MEMBER (gfloat, ffmpegenc, qdata->offset));
       break;
     case G_TYPE_STRING:
-      g_return_val_if_fail (qdata->size == sizeof (gchar*), TRUE);
+      g_return_val_if_fail (qdata->size == sizeof (gchar *), TRUE);
       g_value_take_string (value,
-          g_strdup (G_STRUCT_MEMBER (gchar*, ffmpegenc, qdata->offset)));
+          g_strdup (G_STRUCT_MEMBER (gchar *, ffmpegenc, qdata->offset)));
       break;
-    default: /* must be enum, given the check above */
+    default:                   /* must be enum, given the check above */
       if (G_IS_PARAM_SPEC_ENUM (pspec)) {
         g_return_val_if_fail (qdata->size == sizeof (gint), TRUE);
         g_value_set_enum (value,
@@ -966,7 +956,7 @@ gst_ffmpeg_cfg_get_property (GObject * object,
         g_return_val_if_fail (qdata->size == sizeof (guint), TRUE);
         g_value_set_flags (value,
             G_STRUCT_MEMBER (guint, ffmpegenc, qdata->offset));
-      } else { /* oops, bit lazy we don't cover this case yet */
+      } else {                  /* oops, bit lazy we don't cover this case yet */
         g_critical ("%s does not yet support type %s", GST_FUNCTION,
             g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
       }
@@ -986,8 +976,9 @@ gst_ffmpeg_cfg_set_defaults (GstFFMpegEnc * ffmpegenc)
       &num_props);
 
   for (i = 0; i < num_props; ++i) {
-    GValue val = {0, };
+    GValue val = { 0, };
     GParamSpec *pspec = pspecs[i];
+
     /* only touch those that are really ours; i.e. should have some qdata */
     if (!g_param_spec_get_qdata (pspec, quark))
       continue;
@@ -1022,11 +1013,51 @@ gst_ffmpeg_cfg_fill_context (GstFFMpegEnc * ffmpegenc, AVCodecContext * context)
     context_offset = qdata->offset - CONTEXT_CONFIG_OFFSET;
     if (gst_ffmpeg_cfg_codec_has_pspec (klass->in_plugin->id, pspec)
         && context_offset >= 0) {
-      /* memcpy a bit heavy for a small copy, but hardly part of 'inner loop' */
-      memcpy (G_STRUCT_MEMBER_P (context, context_offset),
-          G_STRUCT_MEMBER_P (ffmpegenc, qdata->offset),
-          qdata->size);
+      if (G_PARAM_SPEC_VALUE_TYPE (pspec) == G_TYPE_STRING) {
+        /* make a copy for ffmpeg, it will likely free only some,
+         * but in any case safer than a potential double free */
+        G_STRUCT_MEMBER (gchar *, context, context_offset) =
+            g_strdup (G_STRUCT_MEMBER (gchar *, ffmpegenc, qdata->offset));
+      } else {
+        /* memcpy a bit heavy for a small copy,
+         * but hardly part of 'inner loop' */
+        memcpy (G_STRUCT_MEMBER_P (context, context_offset),
+            G_STRUCT_MEMBER_P (ffmpegenc, qdata->offset), qdata->size);
+      }
     }
     list = list->next;
   }
 }
+
+void
+gst_ffmpeg_cfg_finalize (GstFFMpegEnc * ffmpegenc)
+{
+  GParamSpec **pspecs;
+  guint num_props, i;
+
+  pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (ffmpegenc),
+      &num_props);
+
+  for (i = 0; i < num_props; ++i) {
+    GParamSpec *pspec = pspecs[i];
+    GParamSpecData *qdata;
+
+    qdata = g_param_spec_get_qdata (pspec, quark);
+
+    /* our param specs should have such qdata */
+    if (!qdata)
+      continue;
+
+    switch (G_PARAM_SPEC_VALUE_TYPE (pspec)) {
+      case G_TYPE_STRING:
+        if (qdata->size == sizeof (gchar *)) {
+          g_free (G_STRUCT_MEMBER (gchar *, ffmpegenc, qdata->offset));
+          G_STRUCT_MEMBER (gchar *, ffmpegenc, qdata->offset) = NULL;
+        }
+        break;
+      default:
+        break;
+    }
+  }
+  g_free (pspecs);
+}
